@@ -27,8 +27,10 @@ import re
 import shutil
 import smtplib
 import subprocess
+import sys
 import threading
 import time
+import traceback
 import urllib
 import urlparse
 
@@ -126,6 +128,13 @@ class ThirdPartyWhitelistEntry(CodeIdentifierBaseClass):
     super(ThirdPartyWhitelistEntry, self).__init__(filename, function_name, code_fragment)
 
 
+class LineTypeEnum(object):
+  DEFAULT = 1
+  KNOWN_ERROR = 2
+  BUILDING_BLOCK = 3
+  THIRDPARTY_WHITELIST = 4
+  IGNORED_FILEPATH = 5
+  BAD_FILEPATH = 6
 
 class FlawlessService(object):
   ############################## CONSTANTS ##############################
@@ -162,6 +171,7 @@ class FlawlessService(object):
 
     self.extract_base_path_pattern = re.compile('^.*/%s/?(.*)$' %
                                                 config.report_runtime_package_directory_name)
+    self.only_blame_patterns = [re.compile(p) for p in config.only_blame_filepaths_matching]
 
     self.lock = threading.RLock()
     self.errors_seen = None
@@ -243,7 +253,7 @@ class FlawlessService(object):
         try:
           task()
         except Exception as e:
-          log.exception(e)
+          self._handle_flawless_issue("<br />".join(traceback.format_exception(*sys.exc_info())))
 
   ############################## Misc Helper Funcs ##############################
 
@@ -255,8 +265,10 @@ class FlawlessService(object):
                          not bool(self.VALIDATE_EMAIL_PATTERN.match(e))]
     if invalid_addresses:
       to_addresses = [e for e in to_addresses if e not in invalid_addresses]
-      log.warning("Invalid email address found. Not sending to: %s" %
-                  ", ".join(invalid_addresses))
+      self._handle_flawless_issue(
+        "Invalid email address found. Not sending to: %s" % ", ".join(invalid_addresses),
+        log_func=log.warning,
+      )
 
     msg = email.MIMEText.MIMEText(body.encode("UTF-8"), "html", "UTF-8")
     msg["From"] = "error_report@%s" % config.email_domain_name
@@ -286,11 +298,14 @@ class FlawlessService(object):
       epoch_ms = int(self.time_func() * 1000)
     return cls.fromtimestamp(epoch_ms / 1000.)
 
-  def _matches_path_list(self, filepath, path_list):
-    '''Given a filepath, and a list of filepath fragments, this function returns true
-    if filepath contains any one of those fragments'''
-    for path in path_list:
-      if path and path in filepath:
+  def _matches_filepath_pattern(self, filepath):
+    '''Given a filepath, and a list of regex patterns, this function returns true
+    if filepath matches any one of those patterns'''
+    if not self.only_blame_patterns:
+      return True
+
+    for pattern in self.only_blame_patterns:
+      if pattern.match(filepath):
         return True
     return False
 
@@ -300,10 +315,35 @@ class FlawlessService(object):
       if entry == e:
         return e
 
-  def _format_traceback(self, request, append_additional_info=True,
-                        linebreak="<br />", spacer="&nbsp;"):
+  def _handle_flawless_issue(self, message, log_func=log.error):
+    log_func(message)
+    if config.default_contact:
+      self._sendmail([config.default_contact], "Unexpected problem on Flawless Server", message)
+
+  def _get_line_type(self, line):
+    match = self.extract_base_path_pattern.match(line.filename)
+    if not match:
+      return LineTypeEnum.BAD_FILEPATH
+
+    filepath = match.group(1)
+    entry = StackTraceEntry(filepath, line.function_name, line.text)
+    if entry in self.third_party_whitelist[filepath]:
+      return LineTypeEnum.THIRDPARTY_WHITELIST
+    elif not self._matches_filepath_pattern(filepath):
+      return LineTypeEnum.IGNORED_FILEPATH
+    elif entry in self.building_blocks[filepath]:
+      return LineTypeEnum.BUILDING_BLOCK
+    elif entry in self.known_errors[filepath]:
+      return LineTypeEnum.KNOWN_ERROR
+    else:
+      return LineTypeEnum.DEFAULT
+
+  def _format_traceback(self, request, append_locals=True, show_full_stack=False,
+                        linebreak="<br />", spacer="&nbsp;", start_bold="<strong>",
+                        end_bold="</strong>"):
+    # Traceback
     parts = []
-    parts.append("Traceback (most recent call last):")
+    parts.append("{b}Traceback (most recent call last):{xb}".format(b=start_bold, xb=end_bold))
     formatted_stack = [
       '{sp}{sp}File "{filename}", line {line}, in {function}{lb}{sp}{sp}{sp}{sp}{code}'.format(
         sp=spacer, lb=linebreak, filename=l.filename, line=l.line_number,
@@ -314,13 +354,30 @@ class FlawlessService(object):
     parts.extend(formatted_stack)
     parts.append(request.exception_message)
 
-    if append_additional_info and request.additional_info:
-      parts.append(linebreak + "Additional Information:")
+    # Frame Locals
+    parts.append(linebreak * 2 + "{b}Stack Frame:{xb}".format(b=start_bold, xb=end_bold))
+    frames_to_show = [l for l in request.traceback if l.frame_locals is not None and
+                      (self._get_line_type(l) in [LineTypeEnum.KNOWN_ERROR, LineTypeEnum.DEFAULT]
+                       or show_full_stack)]
+    for l in frames_to_show:
+      line_info = '{sp}{sp}{b}File "{filename}", line {line}, in {function}{xb}'.format(
+        sp=spacer, filename=l.filename, line=l.line_number, function=l.function_name,
+        b=start_bold, xb=end_bold,
+      )
+      local_vals = ['{sp}{sp}{sp}{sp}{name}={value}'.format(
+                        sp=spacer, name=name, value=value.decode("UTF-8", "replace"))
+                    for name, value in sorted(l.frame_locals.items())]
+      parts.append(line_info)
+      parts.extend(local_vals or [spacer * 4 + "No variables in this frame"])
+
+    # Additional Information
+    if request.additional_info:
+      parts.append(linebreak * 2 + "{b}Additional Information:{xb}".format(b=start_bold, xb=end_bold))
       parts.append(request.additional_info.decode("UTF-8", "replace").replace("\n", linebreak))
 
     return linebreak.join(parts)
 
-  ############################## Public API Funcs ##############################
+  ############################## Record Error ##############################
 
   def record_error(self, request):
       t = self.thread_cls(target=self._record_error, args=[request])
@@ -333,21 +390,17 @@ class FlawlessService(object):
     blamed_entry = None
     email_recipients = []
     for stack_line in traceback:
-      match = self.extract_base_path_pattern.match(stack_line.filename)
-      if match:
-        filepath = match.group(1)
+      line_type = self._get_line_type(stack_line)
+      if line_type == LineTypeEnum.THIRDPARTY_WHITELIST:
+        return None, None, None
+      elif line_type in [LineTypeEnum.DEFAULT, LineTypeEnum.KNOWN_ERROR]:
+        filepath = self.extract_base_path_pattern.match(stack_line.filename).group(1)
         entry = StackTraceEntry(filepath, stack_line.function_name, stack_line.text)
-        if (self._matches_path_list(filepath, config.report_exclude_filepaths_containing) and
-            not self._matches_path_list(filepath, config.report_include_filepaths_containing)):
-          continue
-        elif entry in self.third_party_whitelist[filepath]:
-          return None, None, None
-        elif entry not in self.building_blocks[filepath]:
-          blamed_entry = entry
-          key = api.ErrorKey(filepath, stack_line.line_number,
-                             stack_line.function_name, stack_line.text)
-          if filepath in self.watch_all_errors:
-            email_recipients.extend(self.watch_all_errors[filepath])
+        blamed_entry = entry
+        key = api.ErrorKey(filepath, stack_line.line_number,
+                           stack_line.function_name, stack_line.text)
+        if filepath in self.watch_all_errors:
+          email_recipients.extend(self.watch_all_errors[filepath])
     return (key, blamed_entry, email_recipients)
 
 
@@ -388,7 +441,8 @@ class FlawlessService(object):
       self.errors_seen[key] = err_info
 
     if not err_info:
-      log.warn("Unable to do blame for %s" % str(err_info))
+      self._handle_flawless_issue("Unable to do blame for %s. You may want to consider setting "
+                                  "only_blame_filepaths_matching in your flawless.cfg " % str(key))
       return
 
     # Figure out if we should send an email or not
@@ -414,7 +468,8 @@ class FlawlessService(object):
       if (known_entry.alert_every_n_occurences and
             err_info.error_count % known_entry.alert_every_n_occurences == 0):
         send_email = True
-      if known_entry.max_alert_threshold and err_info.error_count > known_entry.max_alert_threshold:
+      if (known_entry.max_alert_threshold is not None
+          and err_info.error_count > known_entry.max_alert_threshold):
         send_email = False
 
     # Send email if applicable
@@ -449,6 +504,7 @@ class FlawlessService(object):
       err_info.email_sent = True
       self.errors_seen[key] = err_info
 
+  ############################## Summarize Errors ##############################
 
   def index(self, *args, **kwargs):
     return self.get_weekly_error_report(*args, **kwargs)
@@ -523,6 +579,7 @@ class FlawlessService(object):
       </html>
     """.format(data=datastr)
 
+  ############################## Add New Known Error ##############################
 
   def add_known_error(self, filename="", function_name="", code_fragment=""):
     code_fragment = cgi.escape(code_fragment)
